package cambiavalute.persistence;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.util.HashMap;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;

import cambiavalute.model.*;

public class MyCambiUfficialiReader implements CambiUfficialiReader {
	
	public Map<String,TassoDiCambioUfficiale> leggiTabellaCambiUfficiali(Reader input) throws BadFileFormatException {
		
		if (input == null) throw new IllegalArgumentException("Cambi Ufficiali Reader: null reader");
		
		Map<String,TassoDiCambioUfficiale> elencoCambiUfficiali;
		BufferedReader reader = new BufferedReader(input);
		elencoCambiUfficiali = new HashMap<>();
		
		String line = null;
		try {
			while ((line = reader.readLine()) != null) {
				StringTokenizer tokenizer = new StringTokenizer(line, ",");
				String nomePaese = tokenizer.nextToken().trim();
				String nomeValuta = tokenizer.nextToken().trim();
				String siglaValuta = tokenizer.nextToken().trim();
				@SuppressWarnings("unused")	String codicePaese = tokenizer.nextToken().trim();
				String tasso = tokenizer.nextToken().trim();
				// verifiche consistenza formato
				if (siglaValuta.length()!=3) throw new BadFileFormatException("sigla valuta illegale");
				//
				double tassoUfficialeDiCambio = Double.parseDouble(tasso);
				//System.err.println(siglaValuta + "\t" + tassoUfficialeDiCambio);
				TassoDiCambioUfficiale cambioUfficiale= new TassoDiCambioUfficiale(siglaValuta,nomeValuta,nomePaese,tassoUfficialeDiCambio);
				elencoCambiUfficiali.put(siglaValuta,cambioUfficiale);
			}
		} catch (NoSuchElementException | IOException | NumberFormatException e ) {
			throw new BadFileFormatException(e);
		}
		return elencoCambiUfficiali;
	}

	public static void main(String[] args) throws FileNotFoundException, BadFileFormatException{
		CambiUfficialiReader reader = new MyCambiUfficialiReader();
		Map<String,TassoDiCambioUfficiale> mappa = reader.leggiTabellaCambiUfficiali(new BufferedReader(new FileReader("CambiUfficiali.csv")));
		for (String sigla : mappa.keySet()) System.out.println(mappa.get(sigla));
	}
	
}
