package cambiavalute.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.ParseException;
import java.util.Arrays;
import java.util.Map;
import java.util.OptionalDouble;

import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.WindowConstants;

import cambiavalute.model.CambiaValute;
import cambiavalute.ui.controller.Controller;

public class MainFrame extends JFrame implements ActionListener {

	private static final long serialVersionUID = 1L;

	private Controller controller;
	private JPanel mainPanel;
	private JRadioButton acquisto, vendita;
	private ButtonGroup grp;
	
	private JComboBox<String> valutaEsteraCombo;
	private JTextField importoField, cambioUfficialeField;
	private JTextArea outputArea;
	private String[] arrayValuteEstere;
	
	public MainFrame(Controller controller) {
		this.controller = controller;
		arrayValuteEstere = this.controller.getListaValuteEstere().toArray(new String[1]);
		initGUI();
		setSize(450, 300);
	}

	private void initGUI() {

		mainPanel = new JPanel();
		getContentPane().add(mainPanel);

		setTitle("Cambiavalute");
		setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);

		valutaEsteraCombo = new JComboBox<>(arrayValuteEstere);
		mainPanel.add(new JLabel("Valuta"), BorderLayout.NORTH);
		mainPanel.add(valutaEsteraCombo, BorderLayout.NORTH);
		grp = new ButtonGroup();
		acquisto = new JRadioButton("Acquisto"); grp.add(acquisto);
		vendita = new JRadioButton("Vendita"); grp.add(vendita); vendita.setSelected(true);
		mainPanel.add(acquisto, BorderLayout.NORTH);
		mainPanel.add(vendita, BorderLayout.NORTH);
		
		Box box1 = Box.createHorizontalBox();
		importoField = new JTextField(10); importoField.setText("100,00"); importoField.setHorizontalAlignment(JTextField.RIGHT);
		cambioUfficialeField = new JTextField(10);   cambioUfficialeField.setEditable(false); cambioUfficialeField.setHorizontalAlignment(JTextField.RIGHT);
		importoField.setFont(new Font("Courier New", Font.BOLD, 12));
		cambioUfficialeField.setFont(new Font("Courier New", Font.BOLD, 12));
		
		box1.add(new JLabel("Importo "), BorderLayout.NORTH);
		box1.add(importoField, BorderLayout.NORTH);
		box1.add(new JLabel(" Cambio ufficiale "), BorderLayout.NORTH);
		box1.add(cambioUfficialeField, BorderLayout.NORTH);
		mainPanel.add(box1);
		
		outputArea = new JTextArea(12,60); outputArea.setEditable(false); outputArea.setBackground(Color.yellow);
		outputArea.setFont(new Font("Courier New", Font.BOLD, 11));
		mainPanel.add(outputArea, BorderLayout.CENTER);
		
		valutaEsteraCombo.addActionListener(this);
		acquisto.addActionListener(this);
		vendita.addActionListener(this);
		importoField.addActionListener(this);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		outputArea.setText("");
		String valutaEstera = (String) valutaEsteraCombo.getSelectedItem();
		String importoString = (String) importoField.getText();
		double importo;
		try {
			//importo = Double.parseDouble(importoString);
			importo = CambiaValute.convertiInDouble(importoString);
		}
		//catch(NumberFormatException e1){
		catch(ParseException e1){
			controller.getUserInteractor().showMessage("Importo deve essere numerico");
			return;
		}
		
		double tassoUfficiale = controller.getCambiUfficiali().get(valutaEstera).getTassoDiCambio();
		double importoDestinazione = CambiaValute.arrotonda(vendita.isSelected() ? importo * tassoUfficiale : importo / tassoUfficiale);
		importoField.setText(CambiaValute.formatta(importo)); // per estetica
		cambioUfficialeField.setText(CambiaValute.formatta(importoDestinazione));
		
		Map<String,OptionalDouble> cambi = vendita.isSelected() ? controller.changeTo(valutaEstera, importo) : controller.changeFrom(valutaEstera, importo);	
		stampaCambi(cambi, valutaEstera, importo);
	}
	
	private void stampaCambi(Map<String,OptionalDouble> cambi, String siglaValuta, double importo){
		StringBuilder outputBuffer = new StringBuilder();
		outputBuffer.append("Richiesta di compravendita di " + controller.getNomeCompletoValutaEstera(siglaValuta)  +"\n");
		char[] tmp = new char[outputBuffer.length()]; Arrays.fill(tmp, '-');
		String lineaOrizzontale = String.valueOf(tmp)+"\n";	
		for(String agenzia: cambi.keySet()){
			stampaUnCambio(agenzia,outputBuffer, cambi,siglaValuta,importo);
		 }
		outputArea.append(outputBuffer.toString()+lineaOrizzontale); 
	}
	
	private void stampaUnCambio(String agenzia, StringBuilder outputBuffer, Map<String,OptionalDouble> cambi, String siglaValuta, double importo){
			OptionalDouble result = cambi.get(agenzia);
			if (result.isPresent()) {
				outputBuffer.append( agenzia + ": " + CambiaValute.formatta(importo) 
					+ (vendita.isSelected() ? " EUR = " : " " + siglaValuta + " = ") + CambiaValute.formatta(result.getAsDouble())
					+ (acquisto.isSelected() ? " EUR" : " " + siglaValuta) + "\n");
		    } else {
		    	outputBuffer.append( agenzia + ": " + "non tratta la valuta estera richiesta\n" );
		    }
	}	
}
